### ---------------------------------------------------------------------------
# dirbrowser3 - Implements a directory browser, without changing the current
#		directory (permanently) in the current process...
#
#		More info can be found at:
#		    http://r8h.cs.man.ac.uk:8000/tcl/dirbrowser3.html
#
# TO DO
#   1) Reprogram to use option database instead of hardcoded fonts, etc.  
#	     DONE! ONE BIT DOESN'T WORK VERY WELL AT THE MOMENT...
#   2) Adjust to use a separate process to track the current directory
#            MADE AN OPTION, SINCE IT CHANGES THE BEHAVIOUR WITH
#            SYMLINKS SOMETHING ROTTEN...
#   3) Write a cleverer (i.e. atomic) semaphore
#	     NOT NEEDED! WAS GETTING MIXED UP WITH SOMETHING ELSE...
#   4) Update to work with tk4.0
#	     DONE (I think)
#   5) Write a MAN page
#            DONE, NOW IN HTML
#   6) Add filename completion
#            DONE (I think)
#   7) Make it faster (especially to load)
#            ONGOING
#   8) Improve the bindings
#            DONE (I THINK)
#   9) Add drag-n-drop support as a target
#  10) Add drag-n-drop support as a source
#  11) Make the online help into some form of hypertext
#  12) Allow two (or more) simultaneous browsers
#

# This is cunning, using a procedure which deletes itself to provide a
# temporary namespace for initialisation. If this sort of thing is
# obvious to you, then you're quite probably a bit better tcl hacker
# than I am...
;proc dirbrowser_init {} {
    # extended keyboard handling for entries
    foreach binding [bind Listbox] {
	if {
	    [string match *Key* $binding] &&
	    ![string length [bind Entry $binding]]
	} then {
	    regsub -all {\%W} [bind Listbox $binding] {%Wlist} command
	    bind dirbrowser1 $binding $command
	}
    }
    bind dirbrowser2 <Enter> dirbrowser_enterFilter
    bind dirbrowser2 <Leave> {dirbrowser_leaveFilter;dirbrowser_clearsel}

    # Mega-widget defaults
    global auto_index
    set libdir [file dirname [lindex $auto_index(dirbrowser3) 1]]
    option readfile $libdir/dirbrowser.ad 30

    # Don't run me twice!
    rename dirbrowser_init {}
}
dirbrowser_init

;proc bindtagsprepend {w args} {
    bindtags $w [concat $args [bindtags $w]]
}

proc dirbrowser3 {windowname args} {
    global dirbrowser_filename   \
	   dirbrowser
    upvar #0  dirbrowser(buttons)   buttons  \
	      dirbrowser(filter)    filter   \
	      dirbrowser(fileref)   fileref  \
	      dirbrowser(filtcnt)   count    \
              dirbrowser(dirlist)   dirlist  \
              dirbrowser(filelist)  filelist \
	      dirbrowser(toplevel)  w        \
	      dirbrowser(extrabut)  numXbut

    set count 0

    set allowed_switches(-cancelexception) 0
    # set allowed_switches(-dodotfiles)    0
    set allowed_switches(-showhidden)      0
    set allowed_switches(-globalgrab)      0
    set allowed_switches(-usecd)           0
    set allowed_switches(-aftersetuphook)  1
    set allowed_switches(-defaultfile)     1
    set allowed_switches(-extendwidget)    1
    set allowed_switches(-extrabuttons)    1
    set allowed_switches(-hotpaths)        1
    set allowed_switches(-message)         1
    set allowed_switches(-directory)       2
    set allowed_switches(-filemask)        2
    set allowed_switches(-filetest)        2
    set switchval(-directory)              "."
    set switchval(-filemask)               "*"
    set switchval(-filetest)               "1"

    set unused [parseargs allowed_switches switchval $args]
    if [llength $unused] {
	return -code error "unknown option: [lindex $unused 0]"
    }
    if [winfo exists $windowname] {
	return -code error "window already exists: $windowname"
    }

    update idletasks
        
    set w $windowname				;# Really me!
    toplevel $w -class DKFDirbrowser
    set parentwindow [winfo toplevel [winfo parent $w]] ;# My parent toplevel

    # Set up initial directory ------------------------------------------------
    set dirbrowser(usecd) $switchval(-usecd)
    dirbrowser_initdir .
    if [string length $switchval(-defaultfile)] {
	set filedir [file dirname $switchval(-defaultfile)]
	set filename [file tail $switchval(-defaultfile)]
	dirbrowser_chdir $filedir
	# Beware of symlinks to directories!
	set filestats(type) {}
	catch {
	    file stat $filename filestats
	}
	if {$filestats(type) == "directory"} {
	    dirbrowser_chdir $filename
	    set switchval(-defaultfile) {}
	} else {
	    set switchval(-defaultfile) $filename
	}
    } else {
	dirbrowser_chdir $switchval(-directory)
    }
    set switchval(-directory) [dirbrowser_pwd]
    
    # Other initialisation ----------------------------------------------------
    set dirbrowser(filetest)    $switchval(-filetest)
    set dirbrowser(hiddenfiles) $switchval(-showhidden)
    set dirbrowser(hiddendirs)  $switchval(-showhidden)
    set dirbrowser(filterbody)  $switchval(-directory)/$switchval(-filemask)
    
    if [llength $switchval(-extendwidget)] {
	eval $switchval(-extendwidget) $w
    }
    
    wm transient $w $parentwindow
    wm group $w $parentwindow
    
    # References to the principal interactive widgets/widget groups -----------
    
    set buttons   $w.buttons
    set filter    $w.filtertext
    set fileref   $w.filenametext
    set dirlist   ${filter}list
    set filelist  ${fileref}list
    set numXbut   [llength $switchval(-extrabuttons)]
    
    # The frames and labels (ie. the structure) -------------------------------
    
    if [string length $switchval(-message)] {
	pack [message $w.message -justify left -relief raised -bd 2 \
		-text $switchval(-message)] -side top \
		-fill x -expand 1 -anchor w
	catch {
	    dragdrop_destination $w.message accepts_drop FILENAME \
		    dirbrowser_dropFn
	}
    }

    frame $w.filter -relief raised -bd 2
    frame $w.filename -relief raised -bd 2
    frame $w.dirs -relief raised -bd 2 -width 7c
    frame $w.files -relief raised -bd 2 -width 7c
    frame $w.buttons -relief raised -bd 2
    frame $w.buttons.okbrd -relief sunken -bd 1
    frame $w.buttons.filtbrd -relief flat -bd 1
    frame $w.buttons.canbrd -relief flat -bd 1
    for {set i 0} {$i<$numXbut} {incr i} {
	frame $w.buttons.butbrd$i -relief flat -bd 1
    }
    frame $w.buttons.hlpbrd -relief flat -bd 1
    frame $w.filter.body -relief groove -bd 4
    frame $w.filename.body -relief groove -bd 4
    frame $w.dirs.body
    frame $w.dirs.body.1
    frame $w.dirs.body.1.spc
    frame $w.dirs.body.2
    frame $w.files.body
    frame $w.files.body.1
    frame $w.files.body.1.spc
    frame $w.files.body.2
    label $w.filter.label -text "Filter:    "
    label $w.filename.label -text "Filename:    "
    label $w.dirs.label -text "Directory" -anchor e
    label $w.files.label -text "Files" -anchor e

    pack $w.filter -side top -fill x
    pack $w.buttons -side bottom -fill x
    pack $w.filename -side bottom -fill x
    pack $w.files -side right -expand 1 -fill both
    pack $w.dirs -side right -expand 1 -fill both
    pack $w.buttons.okbrd -side left -expand 1 -padx 2m -pady 2m
    pack $w.buttons.filtbrd -side left -expand 1 -padx 2m -pady 2m
    pack $w.buttons.canbrd -side left -expand 1 -padx 2m -pady 2m
    for {set i 0} {$i<$numXbut} {incr i} {
	pack $w.buttons.butbrd$i -side left -expand 1 -padx 2m -pady 2m
    }
    pack $w.buttons.hlpbrd -side left -expand 1 -padx 2m -pady 2m
    pack $w.filter.label -side left
    pack $w.filter.body -padx 1m -pady 1m -expand 1 -fill x
    pack $w.filename.label -side left
    pack $w.filename.body -padx 1m -pady 1m -expand 1 -fill x
    pack $w.dirs.label
    pack $w.dirs.body -expand 1 -fill both -anchor n
    pack $w.dirs.body.2 -side left -fill both -expand 1
    pack $w.dirs.body.1 -side left -fill y
    pack $w.dirs.body.1.spc -side bottom -expand 0
    pack $w.files.label
    pack $w.files.body -expand 1 -fill both -anchor n
    pack $w.files.body.2 -side left -fill both -expand 1
    pack $w.files.body.1 -side left -fill y
    pack $w.files.body.1.spc -side bottom -expand 0

    # The buttons at the bottom of the window ---------------------------------

    button $buttons.ok      -text "O.K."    -command \
	    [list destroy $w]
    button $buttons.cancel  -text "Cancel"  -command \
	    "set dirbrowser_filename {}; destroy $w"
    button $buttons.filter  -text "Filter"  -command \
	    dirbrowser_filterCommand
    button $buttons.help    -text "Help"    -command \
	    [list dirbrowser_helpCommand $w]
    
    pack $buttons.ok -in $buttons.okbrd -side left \
	    -padx 3 -pady 3 -ipadx 3 -ipady 3
    pack $buttons.filter -in $buttons.filtbrd -side left \
	    -padx 3 -pady 3 -ipadx 3 -ipady 3
    pack $buttons.cancel -in $buttons.canbrd -side left \
	    -padx 3 -pady 3 -ipadx 3 -ipady 3
    pack $buttons.help -in $buttons.hlpbrd -side left \
	    -padx 3 -pady 3 -ipadx 3 -ipady 3
    
    for {set i 0} {$i<$numXbut} {incr i} {
	set thisbut [lindex $switchval(-extrabuttons) $i]
	pack [button $buttons.butbrd$i.but -text [lindex $thisbut 0] \
		-command [lindex $thisbut 1]] \
		-side left -padx 3 -pady 3 -ipadx 3 -ipady 3
	bind $buttons.butbrd$i.but <Enter> \
		"dirbrowser_hl $i;dirbrowser_clearsel"
	catch {
	    dragdrop_destination $buttons.butbrd$i.but accepts_drop FILENAME \
		    dirbrowser_dropFn
	}
    }
    
    bind $buttons.ok      <Enter> {dirbrowser_hl ok;dirbrowser_clearsel}
    bind $buttons.okbrd   <Enter> {dirbrowser_hl ok;dirbrowser_clearsel}
    bind $buttons.filter  <Enter> {dirbrowser_hl filter;dirbrowser_clearsel}
    bind $buttons.filtbrd <Enter> {dirbrowser_hl filter;dirbrowser_clearsel}
    bind $buttons.cancel  <Enter> {dirbrowser_hl cancel;dirbrowser_clearsel}
    bind $buttons.canbrd  <Enter> {dirbrowser_hl cancel;dirbrowser_clearsel}
    bind $buttons.help    <Enter> {dirbrowser_hl help;dirbrowser_clearsel}
    bind $buttons.hlpbrd  <Enter> {dirbrowser_hl help;dirbrowser_clearsel}
    bind $buttons         <Leave> {dirbrowser_hl ok;dirbrowser_clearsel}
    
    # The entry area at the top -----------------------------------------------
    
    entry $filter -textvariable dirbrowser(filterbody) \
	    -xscrollcommand "$w.filter.body.scrl set"
    scrollbar $w.filter.body.scrl -command "$filter xview" \
	    -orient horizontal  -takefocus 0
    
    pack $w.filter.body.scrl -side bottom -fill x -expand 1
    pack $filter -in $w.filter.body -fill x -expand 1
    
    bindtagsprepend $w.filter.body.scrl dirbrowser2
    bindtagsprepend $filter dirbrowser2
    bind $w.filter <Motion> {dirbrowser_hl filter}
    
    # The entry area at the bottom --------------------------------------------
    
    entry $fileref -textvariable dirbrowser(filebody) \
	    -xscrollcommand "$w.filename.body.scrl set"
    scrollbar $w.filename.body.scrl -command "$fileref xview" \
	    -orient horizontal  -takefocus 0
    set dirbrowser(filebody) "$switchval(-directory)/$switchval(-defaultfile)"
    
    pack $w.filename.body.scrl  -side bottom  -fill x
    pack $fileref -in $w.filename.body -fill both -expand 1
    
    # The directory list at the side ------------------------------------------
    
    scrollbar $w.dirsYScroll -command "$dirlist yview" -takefocus 0
    scrollbar $w.dirsXScroll -command "$dirlist xview" \
	    -orient horizontal  -takefocus 0
    listbox $dirlist -relief raised -bd 2 -takefocus 0 \
	    -yscrollcommand "$w.dirsYScroll set" \
	    -xscrollcommand "$w.dirsXScroll set" \
	    -exportselection 0
    checkbutton $w.dirs.hidden -text "Show Hidden Directories" -highlightt 0 \
	    -var dirbrowser(hiddendirs) -command dirbrowser_filterCommand \
	    -padx 0 -pady 0 -takefocus 0
    
    pack $w.dirsXScroll -in $w.dirs.body.2 -side bottom -fill x
    pack $w.dirsYScroll -in $w.dirs.body.1 -fill y -expand 1
    pack $dirlist -in $w.dirs.body.2 -side left -fill both -expand 1
    pack $w.dirs.hidden -fill x
    $w.dirs.body.1.spc configure -height [expr	\
	    [  $w.dirsXScroll cget -width] +	\
	    2*[$w.dirsXScroll cget -bd]    +	\
	    2*[$w.dirsXScroll cget -highlightthickness]]
    
    bind $dirlist <Double-Button-1> \
	    {dirbrowser_filterCommand [%W get [%W curselection]]}
    bindtagsprepend $w.dirs dirbrowser2
    bindtagsprepend $w.dirsXScroll dirbrowser2
    bindtagsprepend $w.dirsYScroll dirbrowser2
    bindtagsprepend $dirlist dirbrowser2

    # The file list at the other side -----------------------------------------

    scrollbar $w.filesYScroll -command "$filelist yview" -takefocus 0
    scrollbar $w.filesXScroll -command "$filelist xview" \
	    -orient horizontal  -takefocus 0
    listbox $filelist -relief raised -bd 2 -takefocus 0	\
	    -yscrollcommand "$w.filesYScroll set" \
	    -xscrollcommand "$w.filesXScroll set" \
	    -exportselection 0
    checkbutton $w.files.hidden -text "Show Hidden Files" -highlightt 0 \
	    -var dirbrowser(hiddenfiles) -command dirbrowser_filterCommand \
	    -padx 0 -pady 0 -takefocus 0

    set dirbrowser_filename $switchval(-directory)

    pack $w.filesXScroll -in $w.files.body.2 -side bottom -fill x
    pack $w.filesYScroll -in $w.files.body.1 -fill y -expand 1
    pack $filelist -in $w.files.body.2 -side left -fill both -expand 1
    pack $w.files.hidden -fill x
    $w.files.body.1.spc configure -height [expr	\
	    [  $w.filesXScroll cget -width] +	\
	    2*[$w.filesXScroll cget -bd]    +	\
	    2*[$w.filesXScroll cget -highlightthickness]]
    
    bind $filelist <ButtonRelease-1> {
	set dirbrowser_filename [%W get [%W curselection]]
	set dirbrowser(filebody) \
		[file dirname $dirbrowser(filebody)]/$dirbrowser_filename
	tkEntrySeeInsert $dirbrowser(fileref)
    }
    bind $filelist <Double-Button-1> {
	set dirbrowser_filename [%W get [%W curselection]]
	$dirbrowser(buttons).ok invoke
	break
    }
    
    # More bindings.
    bindtagsprepend $filter dirbrowser1
    bindtagsprepend $fileref dirbrowser1
    bind $w <Escape> [list $buttons.cancel invoke]
    bind $fileref <Return> [list $buttons.ok invoke]
    bind $filter <Return> dirbrowser_filterCommand
    bind $fileref <Any-Key> dirbrowser_filekey
    bind $filter <Any-Key> dirbrowser_dirkey
    if [llength $switchval(-hotpaths)] {
	bind $w <3> [list dirbrowser_do_menu $w $switchval(-hotpaths) %X %Y]
    }
    bind $w <F1> [list dirbrowser_helpCommand $w]
    catch {
	bind $w <Help> [list dirbrowser_helpCommand $w]
    }
    bind $fileref <Tab> {
	if [dirbrowser_complete] {
	    dirbrowser_filterCommand
	    break
	} else {
	    dirbrowser_filterCommand
	}
    }

    # Can't be bothered to bind anything except the main widget
    catch {
	dragdrop_destination $fileref accepts_drop FILENAME dirbrowser_dropFn
	dragdrop_destination $filter accepts_drop FILENAME dirbrowser_dropFn
	dragdrop_destination $filelist accepts_drop FILENAME dirbrowser_dropFn
	dragdrop_destination $dirlist accepts_drop FILENAME dirbrowser_dropFn
    }

    if [llength $switchval(-aftersetuphook)] {
	eval $switchval(-aftersetuphook) $w
    }    

    dirbrowser_filterCommand .
    
    # Only return when the user has supplied a value --------------------------
    
    wm withdraw $w
    update idletasks
    wm title $w "Directory Browser"
    catch {
	regexp {([-+][0-9]+)([-+][0-9]+)} \
		[wm geometry $parentwindow] geom geomX geomY
	wm geometry $w +[expr 30 $geomX]+[expr 30 $geomY]
    }
    wm deiconify $w
    
    tkwait visibility $w
    if $switchval(-globalgrab) {
	with_grab_focus $w $fileref -global "tkwait window $w"
    } else {
	with_grab_focus $w $fileref         "tkwait window $w"
    }

    if [string length $dirbrowser_filename] {
	return $dirbrowser(filebody)
    } elseif $switchval(-cancelexception) {
	return -code 64 "File Selection Cancelled"
    } else {
	return {}
    }
}

### ---------------------------------------------------------------------------
# Highlighting
;proc dirbrowser_hl which {
    global dirbrowser

    switch $which {
	ok {
	    $dirbrowser(buttons).okbrd   configure -relief sunken
	    $dirbrowser(buttons).filtbrd configure -relief flat
	    $dirbrowser(buttons).canbrd  configure -relief flat
	    $dirbrowser(buttons).hlpbrd  configure -relief flat
	    focus $dirbrowser(fileref)
	}
	filter {
	    $dirbrowser(buttons).okbrd   configure -relief flat
	    $dirbrowser(buttons).filtbrd configure -relief sunken
	    $dirbrowser(buttons).canbrd  configure -relief flat
	    $dirbrowser(buttons).hlpbrd  configure -relief flat
	    focus $dirbrowser(filter)
	}
	cancel {
	    $dirbrowser(buttons).okbrd   configure -relief flat
	    $dirbrowser(buttons).filtbrd configure -relief flat
	    $dirbrowser(buttons).canbrd  configure -relief sunken
	    $dirbrowser(buttons).hlpbrd  configure -relief flat
	    focus $dirbrowser(buttons).cancel
	}
	help {
	    $dirbrowser(buttons).okbrd   configure -relief flat
	    $dirbrowser(buttons).filtbrd configure -relief flat
	    $dirbrowser(buttons).canbrd  configure -relief flat
	    $dirbrowser(buttons).hlpbrd  configure -relief sunken
	    focus $dirbrowser(buttons).help
	}
	default {
	    $dirbrowser(buttons).okbrd   configure -relief flat
	    $dirbrowser(buttons).filtbrd configure -relief flat
	    $dirbrowser(buttons).canbrd  configure -relief flat
	    $dirbrowser(buttons).hlpbrd  configure -relief flat
	    for {set i 0} {$i<$dirbrowser(extrabut)} {incr i} {
		if {$i==$which} {
		    $dirbrowser(buttons).butbrd$i configure -relief sunken
		} else {
		    $dirbrowser(buttons).butbrd$i configure -relief flat
		}
	    }
	    focus $dirbrowser(buttons).butbrd$which.but
	    return
	}
    }
    for {set i 0} {$i<$dirbrowser(extrabut)} {incr i} {
	$dirbrowser(buttons).butbrd$i configure -relief flat
    }
}

# Increments the flag indicating if in the filter part, changing the
# focus(etc.) if I'm in for real!
;proc dirbrowser_enterFilter {} {
    global dirbrowser

    incr dirbrowser(filtcnt)
    if {$dirbrowser(filtcnt) > 0} {
	dirbrowser_hl filter
    }
}

# Decrements the flag indicating if in the filter part, changing the focus(etc.) if I'm out for real!
;proc dirbrowser_leaveFilter {} {
    global dirbrowser

    incr dirbrowser(filtcnt) -1
    if {$dirbrowser(filtcnt) <= 0} {
	set dirbrowser(filtcnt) 0
	dirbrowser_hl ok
    }
}

### ---------------------------------------------------------------------------
# Major directory browser actions

;proc dirbrowser_complete {} {
    global dirbrowser

    set mask [file tail $dirbrowser(filterbody)]
    set startingpoint [file tail $dirbrowser(filebody)]
    set directory [file dirname $dirbrowser(filebody)]

    foreach d [dirbrowser_getdirlist $directory] {
	if ![string first $dirbrowser(filebody) $d] {
	    lappend expands $d
	}
    }
    foreach f [dirbrowser_getfilelist $directory $mask] {
	if ![string first $dirbrowser(filebody) $f] {
	    lappend expands $f
	}
    }
    switch [llength $expands] {
	0 {
	    return 0
	}
	1 {
	    set dirbrowser(filebody) [lindex $expands 0]
	    set dirbrowser(filterbody) \
		    "[file dirname $dirbrowser(filebody)]/$mask"
	    return 1
	}
	default {
	    set flag 0
	    set successflag 0
	    set matchedflag 0
	    set mlen [string length $dirbrowser(filebody)]
	    while 1 {
		set match [string range [lindex $expands 0] 0 $mlen]
		foreach d $expands {
		    if [string first $match $d] {
			set flag 1
		    } elseif ![string compare $match $d] {
			set matchedflag 1
		    }
		}
		if $flag {
		    # No longer prefix
		    return $successflag
		} else {
		    set dirbrowser(filebody) $match
		    set dirbrowser(filterbody) \
			    "[file dirname $dirbrowser(filebody)]/$mask"
		    if $matchedflag {
			return 1
		    }
		    set successflag 1
		    incr mlen
		}
	    }
	}
    }
}

# Code to update the contents of the various buffers when the
# filtering changes.  This is the real key to this whole file!
;proc dirbrowser_filterCommand {{dirname {}}} {
    global dirbrowser

    if {![string compare $dirname "<< Parent >>"]} {
	dirbrowser_chdir ..
    } elseif {[string compare $dirname {}] && $dirname != "."} {
	dirbrowser_chdir $dirname
    } else {
	dirbrowser_chdir [file dirname $dirbrowser(filterbody)]
    }

    set directory [dirbrowser_pwd]
    set filemask [file tail $dirbrowser(filterbody)]
    set dirbrowser(filterbody) "$directory/$filemask"
    set dirbrowser(filebody) "$directory/[file tail $dirbrowser(filebody)]"

    $dirbrowser(dirlist) delete 0 end
    if {!([file dirname $directory]=="$directory")} {
	$dirbrowser(dirlist) insert end "<< Parent >>"
    }
    foreach dir [dirbrowser_getdirlist $directory] {
	regexp {/([^/]*)/$} $dir foo dir
	if {!($dir == ".") && !($dir == "..")} {
	    $dirbrowser(dirlist) insert end $dir
	}
    }

    $dirbrowser(filelist) delete 0 end
    foreach file [dirbrowser_getfilelist $directory $filemask] {
	$dirbrowser(filelist) insert end [file tail $file]
    }
    catch {
	tkEntrySeeInsert $dirbrowser(filter)
	tkEntrySeeInsert $dirbrowser(fileref)
    }
}

;proc dirbrowser_clearsel {} {
    global dirbrowser
    $dirbrowser(dirlist) selection clear 0 end
    $dirbrowser(filelist) selection clear 0 end
}

;proc dirbrowser_filekey {} {
    global dirbrowser
    set w $dirbrowser(filelist)
    if [llength [$w curselection]] {
	set dirbrowser(filebody) \
		[file dirname $dirbrowser(filebody)]/[$w get [$w curselection]]
	tkEntrySeeInsert $dirbrowser(fileref)
    }
}
;proc dirbrowser_dirkey {} {
    global dirbrowser
    set w $dirbrowser(dirlist)
    if [llength [$w curselection]] {
	set fb $dirbrowser(filterbody)
	set sel [$w get [$w curselection]]
	if {$sel == "<< Parent >>"} {
	    set dirbrowser(filterbody) \
		    [file dirname [dirbrowser_pwd]]/[file tail $fb]
	} else {
	    set dirbrowser(filterbody) \
		    [dirbrowser_pwd]/$sel/[file tail $fb]
	}
	tkEntrySeeInsert $dirbrowser(filter)
    }
}

;proc dirbrowser_dropFn {W X Y src ty data} {
    global dirbrowser
    set dirbrowser(filebody) $data
    set dirbrowser(filterbody) \
	    [file dirname $data]/[file tail $dirbrowser(filterbody)]
}

;proc dirbrowser_helpCommand {w} {
    tk_dialog $w.help "Directory Browser Help" "The directory browser\
	    is split into two main regions:\n      - the Filter/Directory\
	    Area, and\n      - the File area.\n\n  In the Filter Area, you\
	    can type in the name of the directory you wish to change\
	    to and the filename filter you wish to apply there. Pressing\
	    Return (or the Filter button) causes the browser to use the\
	    values you have just entered. Double-clicking on one of the\
	    directory names listed in the list at the side will cause the\
	    system to change into that directory.\n\n  In the File Area,\
	    you may type in the name of the file that you want (pressing\
	    Return or the O.K. button to accept what you have typed in).\
	    Double-clicking on one of the listed files will cause that to\
	    be selected as the required file, and single clicking will\
	    transfer that file to the filename entry box.\n\n  Pressing\
	    Escape anywhere cancels, and pressing the right mouse button\
	    brings up a hot directory list (if one is available). This\
	    help may be brought up by pressing F1 or Help." info 0 {O.K.}    
}
option add *WrapLength 5i 100 ;# Ugh!
